#!/usr/bin/env python
# Storm mini Evolution clone by Damien Tanner (dctanner@xoasis.com)      
#
# This file is part of Storm.
#
#    Storm is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Storm is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Storm; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from gtk import *
from modules.xmlparse import *
from modules.storm_constants import *
import string, os, time

class Calendar:
	def __init__(self, parent):
		self.parent = parent
		self.cmap = self.parent.window.get_colormap()
		date = time.strftime("%d %m %Y", time.localtime(time.time()))
		self.cal_date = string.split(date)
		self.cal_time = ['00','00']

		frame=GtkFrame()
		frame.set_usize(-1, -1)
		frame.set_shadow_type("GTK_SHADOW_ETCHED_IN")
		frame.set_label("Calendar")
		frame.set_label_align(0, -1)
		frame.set_name("frame")
		rc_parse_string('widget "*frame*" style "storm_normal"') 
		self.frame=frame
		frame.show()

		vbox = GtkVBox()
		frame.add(vbox)
		vbox.show()

		hbox = GtkHBox()
		vbox.pack_start(hbox, expand=FALSE)
		hbox.show()

		# make the toobar
		toolbar2 = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		toolbar2.set_button_relief(RELIEF_NONE)
		toolbar2.set_space_size(10)
		hbox.pack_start(toolbar2, fill=TRUE)
		# add the evolution pixmaps to the toolbar
		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/new.xpm")
		toolbar2.append_item(None, 'Create a new appointment', None, GtkPixmap(pix, mask), self.new_appointment)

		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/edit.xpm")
		toolbar2.append_item(None, 'Edit the selected appointment', None, GtkPixmap(pix, mask), self.edit_appointment)

		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/trash.xpm")
		toolbar2.append_item(None, 'Delete the selected appointment', None, GtkPixmap(pix, mask), self.delete_appointment)

		toolbar2.show()

		# make the toobar
		toolbar3 = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		toolbar3.set_button_relief(RELIEF_NONE)
		toolbar3.set_space_size(10)
		hbox.pack_start(toolbar3, fill=TRUE)
		# add the evolution pixmaps to the toolbar
		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/left.xpm")
		toolbar3.append_item(None, 'Back 24 hours', None, GtkPixmap(pix, mask), self.back_one_day)

		self.date_label = GtkLabel(time.strftime("%a, %b %d %Y", time.strptime(time.strftime("%b %d %Y", (int(self.cal_date[c_DateYear]), int(self.cal_date[c_DateMonth]), int(self.cal_date[c_DateDay]), 0, 0, 0, 0, 0, 0)), "%b %d %Y")))
		date_button = GtkButton()
		date_button.set_relief(RELIEF_NONE)
		date_button.add(self.date_label)
		date_button.connect("clicked", self.select_date_from_calendar)
		toolbar3.append_widget(date_button, 'Select Date from Calendar', '')
		self.date_label.show()
		date_button.show()

		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/right.xpm")
		toolbar3.append_item(None, 'Forward 24 hours', None, GtkPixmap(pix, mask), self.forward_one_day)


		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/home.xpm")
		toolbar3.append_item(None, 'Today', None, GtkPixmap(pix, mask), self.today_date)

		toolbar3.show()

		cscroll = GtkScrolledWindow()
		cscroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		vbox.pack_start(cscroll)
		cscroll.show()

		clist=GtkCList(3)
		cscroll.add(clist)
		clist.connect("select-row" , self.select_appointment)
		clist.set_flags(CAN_FOCUS)
		clist.set_usize(-1, 87)
		clist.set_column_title(0, "Time")
		clist.set_column_title(1, "Alarm")
		clist.set_column_title(2, "Summary")
		clist.set_column_width(0, 30)
		clist.set_column_width(1, 10)
		clist.set_column_width(2, 90)
		clist.set_selection_mode(SELECTION_SINGLE)
		clist.column_titles_hide()
		clist.show()
		self.clist = clist

		scroll = GtkScrolledWindow()
		scroll.set_policy(POLICY_NEVER, POLICY_ALWAYS)
		self.scroll = scroll
		vbox.pack_start(scroll)
		self.text_scroll = scroll

		text = GtkText()
		self.text = text
		text.set_editable(FALSE)
		scroll.add(text)
		text.show()
		self.text = text

		self.selected_appointment = -1
		self.selected_cal_row = 0

		v = '00'
		j = 0

		# Add a list times for every 30 minutes of the day
		while j < 24:
			# y = string.rjust(str(j), 2).replace(' ', '0') + ':' + v
			# make the above work for python 1.5:
			y = string.rjust(str(j), 2)
			y = string.replace(y, ' ', '0') + ':' + v

			self.clist.append([y, '', ''])

			if v == '00':
				v = '30'
			elif v == '30':
				v = '00'
				j = j + 1

		# Read the appointments xml file
		xml = XmlParse()
		self.xml_appointments = xml.parse_xml_file(self.parent.storm_userdatadir + 'appointments.xml', 'appointment')
		self.appt = []

#		["summary",
#		 ["start-time-hour",
#		  "start-time-minute"],
#		 ["end-time-hour",
#		  "end-time-minute"],
#		 ["date-day",
#		  "date-month",
#		  "date-year"],
#		 ["alarm-status",
#		  "alarm-id"],
#		 "description"]

		for xml_appointment in self.xml_appointments:
			self.appt.append([xml.xml_get_text(xml_appointment, "summary"),[xml.xml_get_text(xml_appointment, "start-time-hour"), xml.xml_get_text(xml_appointment, "start-time-minute")],[xml.xml_get_text(xml_appointment, "end-time-hour"), xml.xml_get_text(xml_appointment, "end-time-minute")],[xml.xml_get_text(xml_appointment, "date-day"), xml.xml_get_text(xml_appointment, "date-month"), xml.xml_get_text(xml_appointment, "date-year")],[xml.xml_get_text(xml_appointment, "alarm-status"), xml.xml_get_text(xml_appointment, "alarm-id")], xml.xml_get_text(xml_appointment, "description")])

	def select_appointment(self, *args):
		'''This is called when the user clicks on an appointment from the list. It then adds the info to a GtkText.'''

		self.selected_cal_row = args[1]
		self.selected_appointment = self.clist_ref[args[1]]
		cal_time_text = string.split(self.clist.get_text(args[1],0), ":")
		self.cal_time = string.split(self.clist.get_text(args[1],0), ":")

		if self.selected_appointment != -1:
			self.text.delete_text(0,-1)
			# Add a check here to see if the two
			# displayed times are not the same (don't
			# display the 2nd if this is the case) [CM]:
			#
			# And it would look so cool if we had bold/italic/colored text here...
			self.text.insert(None,None,None, '[' + self.appt[self.selected_appointment][1][0] + ':' + self.appt[self.selected_appointment][1][1] + ' - ' + self.appt[self.selected_appointment][2][0] + ':' + self.appt[self.selected_appointment][2][1] + ']\n' + self.appt[self.selected_appointment][0] + "\n" + self.appt[self.selected_appointment][5])
			self.text_scroll.show()
		else:
			self.text_scroll.hide()

	def update_list(self):
		'''Update the CList with appointments.'''

		# Set some gdk colors and varibles for looping
		clist_color = self.cmap.alloc("grey87");
		clist_color_dark = self.cmap.alloc("grey80");
		clist_color_white = self.cmap.alloc("white");
		self.clist_ref = []

		HalfHours = 0
		b = 0
		q = 0
		title = 1
		blank = 0

		# For every 30 minutes of a 24 hour day:
		# check if there's an appointment scheduled then
		while HalfHours < 48:
			# More looping variables
			ApptIdx = 0
			stop = 0
			blank = 1

			# Loop through every appointment and see
			# if it's scheduled for the current time
			# in the loop
			while ApptIdx < len(self.appt):

				if self.appt[ApptIdx][c_Date][c_DateDay]   == self.cal_date[c_DateDay]   and \
				   self.appt[ApptIdx][c_Date][c_DateMonth] == self.cal_date[c_DateMonth] and \
				   self.appt[ApptIdx][c_Date][c_DateYear]  ==  self.cal_date[c_DateYear] and \
				   stop == 0:

					# Work out exactly how many
					# minutes since 00:00 it is
					# supposed to start and end
					start_min = int(self.appt[ApptIdx][c_StartTime][c_StartTimeHour]) * 60
					end_min   = int(self.appt[ApptIdx][c_EndTime][c_EndTimeHour]) * 60
					start_min = start_min + int(self.appt[ApptIdx][c_StartTime][c_StartTimeMin])
					end_min   = end_min   + int(self.appt[ApptIdx][c_EndTime][c_EndTimeMin])

					# Make sure it falls between the current loop time
					if start_min - 29 <= q and end_min >= q:

						# Check if this is the first row of the appointment: if so make it colored darker and add a title
						if title == 1:
							self.clist.set_text(HalfHours,2,self.appt[ApptIdx][c_Summary])
							self.clist.set_background(HalfHours, clist_color_dark)
							self.clist_ref.append(ApptIdx)

							# If the appointment has an alarm set show a little xpm
							if self.appt[ApptIdx][c_Alarm][c_AlarmStatus] == 'yes':
								pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/alarm.xpm")
								alarm_pixmap = GtkPixmap(pix, mask)
								self.clist.set_pixmap(HalfHours, 1, alarm_pixmap)
							else:
								pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/alarm-shaded.xpm")
								alarm_pixmap = GtkPixmap(pix, mask)
								self.clist.set_pixmap(HalfHours, 1, alarm_pixmap)

							title = 0
							blank = 0
							#stop = 1
						# If not just color it a light grey
						else:
							self.clist.set_background(HalfHours, clist_color)
							self.clist_ref.append(ApptIdx)
							blank = 0
							#stop = 1

					# If the next loop round
					# will be the end of this
					# current appointment make
					# sure the next one has a
					# title etc...
				        elif end_min + 30 >= q and end_min - 30 <= q:
						title = 1
				ApptIdx = ApptIdx + 1

			# If there's no text for the current loop
			# time: just leave it alone (not that I'm
			# actualy going to leave it alone)
			if blank == 1:

				# Remove any previous xpms or summary text
				self.clist.set_text(HalfHours,1,'')
				self.clist.set_text(HalfHours,2,'')
				self.clist.set_background(HalfHours, clist_color_white)
				self.clist_ref.append(-1)
				title = 1

			HalfHours = HalfHours + 1
			q = q + 30

		# Select the last row we selected
		self.clist.select_row(self.selected_cal_row,0)

	def new_appointment(self, *args):
		'''Open up a new appointment window.'''
		self.do_appointment("new")

	def edit_appointment(self, *args):
		'''Edit the currently selected appointment.'''
		self.do_appointment("edit")

	def delete_appointment(self, *args):
		'''Delete the currently selected appointment.'''

		# If an alarm was scheduled: make sure we delete it
		if self.appt[self.selected_appointment][c_Alarm][c_AlarmStatus] == 'yes':
			os.system("uschedulerm " + self.appt[self.selected_appointment][c_Alarm][c_AlarmId])

		# Delete the actualy appointment infomation
		del self.appt[self.selected_appointment]

		# Update the xml, list and today
		self.update_xml()
		self.update_list()
		self.parent.today.update_today()

	def save_appointment(self, *args):
		'''Saves the just created appointment.'''

		# should do full sanity check of the appointment here (like "time values are just integers" etc.) [CM]
		if len(self.summary_entry.get_text()) == 0:
			self.summary_entry.set_text("<no summary>")
			# check for minutes not == ":00" or ":30" [CM]:
#			self.summary_entry.set_text(string.rjust("[" + self.start_time_combo_hour.entry.get_text(), 2).replace(' ', '0') + ":" + string.rjust(self.start_time_combo_minute.entry.get_text(), 2).replace(' ', '0') + "] <no summary>")

		# If the alarm box is ticked: schedule it with uschedule and get the alarm id
		if self.alarm_check.get_active() == TRUE:
			alarm_status = 'yes'
			usched_id = string.split(os.popen("uschedulecmd /usr/bin/storm-alarm").read())
			alarm_id  = usched_id[7]

			# uschedule wants everything in UTC (time.gmtime)...
			# ... so get the local time for the appointment
			appt_localtime = self.cal_date[c_DateYear] + "-" + self.cal_date[c_DateMonth] + "-" + self.cal_date[c_DateDay] + " " \
			+ string.rjust(self.start_time_combo_hour.entry.get_text(), 2).replace(' ', '0') + ":" \
			+ string.rjust(self.start_time_combo_minute.entry.get_text(), 2).replace(' ', '0') + ":00"

			#os.system("uschedule " + alarm_id + " '" + self.cal_date[c_DateYear] + "-" + self.cal_date[c_DateMonth] + "-" + self.cal_date[c_DateDay] + " " + string.rjust(self.start_time_combo_hour.entry.get_text(), 2).replace(' ', '0') + ":" + string.rjust(self.start_time_combo_minute.entry.get_text(), 2).replace(' ', '0') + ":00'")
			
			print "Appointment is at " + appt_localtime

			# parse the local time string to the seconds since the Epoch:
                        appt_gmtime_tmp = time.mktime(time.strptime(appt_localtime, "%Y-%m-%d %T"))
                        print "appt_gmtime_tmp is " + time.strftime("%Y-%m-%d %T", time.gmtime(appt_gmtime_tmp))

			# create the string again, just in UTC this time:
                        appt_gmtime = time.strftime("%Y-%m-%d %T", time.gmtime(appt_gmtime_tmp))

			# build the uschedule command line:
                        usched_cmd = "uschedule " + alarm_id + " '" + appt_gmtime + "'"
                        print "usched_cmd is " + usched_cmd

			# run uschedule finally:
                        os.system(usched_cmd)
			
		else:
			alarm_status = 'no'
			alarm_id = ''

		# Add the info for the new alarm
		self.appt.append([self.summary_entry.get_text(), [self.start_time_combo_hour.entry.get_text(), self.start_time_combo_minute.entry.get_text()], [self.end_time_combo_hour.entry.get_text(), self.end_time_combo_minute.entry.get_text()], [self.cal_date[c_DateDay], self.cal_date[c_DateMonth], self.cal_date[c_DateYear]], [alarm_status, alarm_id], self.description_text.get_chars(0, -1)])

		# Update the xml, list and today
		self.update_xml()
		self.update_list()
		self.appt_window.hide()
		self.parent.today.update_today()

	def save_edited_appointment(self, *args):
		'''Saves and edited appointment.'''

		# should do full sanity check of the appointment here (like "time values are just integers" etc.) [CM]
		if len(self.summary_entry.get_text()) == 0:
			self.summary_entry.set_text("<no summary>")

		# Replace the old values with the new ones
		self.appt[self.selected_appointment][c_Summary]                    = self.summary_entry.get_text()
		self.appt[self.selected_appointment][c_StartTime][c_StartTimeHour] = self.start_time_combo_hour.entry.get_text()
		self.appt[self.selected_appointment][c_StartTime][c_StartTimeMin]  = self.start_time_combo_minute.entry.get_text()
		self.appt[self.selected_appointment][c_EndTime][c_EndTimeHour]     = self.end_time_combo_hour.entry.get_text()
		self.appt[self.selected_appointment][c_EndTime][c_EndTimeMin]      = self.end_time_combo_minute.entry.get_text()
		self.appt[self.selected_appointment][c_Description]                = self.description_text.get_chars(0, -1)

		# If there was a previously scheduled alarm, delete it first
		if self.appt[self.selected_appointment][c_Alarm][c_AlarmStatus] == 'yes':
			os.system("uschedulerm " + self.appt[self.selected_appointment][c_Alarm][c_AlarmId])

		# Schedule the alarm if needed
		if self.alarm_check.get_active() == TRUE:
			# Get the uschedule command id
			usched_id = string.split(os.popen("uschedulecmd /usr/bin/storm-alarm").read())
			self.appt[self.selected_appointment][c_Alarm][c_AlarmId] = usched_id[7]

			# uschedule wants everything in UTC (time.gmtime)...
			# ... so get the local time for the appointment
			appt_localtime = \
			self.cal_date[c_DateYear] + "-" + self.cal_date[c_DateMonth] + "-" + self.cal_date[c_DateDay] + " " \
			+ string.rjust(self.start_time_combo_hour.entry.get_text(), 2).replace(' ', '0') + ":" \
			+ string.rjust(self.start_time_combo_minute.entry.get_text(), 2).replace(' ', '0') + ":00"
			
			print "Appointment is at " + appt_localtime

			# parse the local time string to the seconds since the Epoch:
                        appt_gmtime_tmp = time.mktime(time.strptime(appt_localtime, "%Y-%m-%d %T"))
                        print "appt_gmtime_tmp is " + time.strftime("%Y-%m-%d %T", time.gmtime(appt_gmtime_tmp))

			# create the string again, just in UTC this time:
                        appt_gmtime = time.strftime("%Y-%m-%d %T", time.gmtime(appt_gmtime_tmp))

			# build the uschedule command line:
                        usched_cmd = "uschedule " + self.appt[self.selected_appointment][c_Alarm][c_AlarmId] + " '" + appt_gmtime + "'"
			print "usched_cmd is " + usched_cmd

			# run uschedule finally:
                        os.system(usched_cmd)
			
			self.appt[self.selected_appointment][c_Alarm][c_AlarmStatus] = 'yes'
		else:
			self.appt[self.selected_appointment][c_Alarm][c_AlarmStatus] = 'no'
			self.appt[self.selected_appointment][c_Alarm][c_AlarmId] = ''

		# Update the xml, list and today
		self.update_xml()
		self.update_list()
		self.select_appointment(None, self.selected_appointment)
		self.appt_window.hide()
		self.parent.today.update_today() 

	def do_appointment(self, type):
		'''Open up the appointment new/edit window.'''

		# If the currently selected time is blank make a new appointment
		if self.selected_appointment == -1:
			type = "new"

		# our main gtk window
		window = GtkWindow(WINDOW_TOPLEVEL)
		self.appt_window = window
		window.connect("delete_event", window.hide)
		window.set_title("Storm - Appointment")
		if screen_width() > 300:
			window.set_usize(280, 240)
		else:
			window.set_usize(240, 280)
		window.set_name("window")
		rc_parse_string('widget "*window*" style "storm_normal"') 
		window.show()

		vbox = GtkVBox()
		window.add(vbox)
		vbox.show()

		# make the toobar
		toolbar = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		toolbar.set_button_relief(RELIEF_NONE)
		toolbar.set_space_size(10)
		vbox.pack_start(toolbar, expand=FALSE)
		toolbar.show()
		# add the evolution pixmaps to the toolbar
		if type == "edit":
			pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/save.xpm")
			toolbar.append_item(None, 'Save the current appointment', None, GtkPixmap(pix, mask), self.save_edited_appointment)
		else:
			pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/save.xpm")
			toolbar.append_item(None, 'Save the current appointment', None, GtkPixmap(pix, mask), self.save_appointment)

		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/cancel.xpm")
		toolbar.append_item(None, 'Close this window', None, GtkPixmap(pix, mask), window.hide)


		# Make a list of hours and minutes for time selection
		time_hours = []
		time_minutes = []
		t = 0
		for t in range (1,24):
			time_hours.append(string.rjust(str(t), 2).replace(' ', '0'))
		t = 0
		while t < 60:
			time_minutes.append(string.rjust(str(t), 2).replace(' ', '0'))
			t = t + 5

		table1=GtkTable(5, 2)
		vbox.pack_start(table1)
		table1.show()
		table1.set_usize(-1, -1)
		table1.set_row_spacings(0)
		table1.set_col_spacings(0)
		label1=GtkLabel()
		table1.attach(label1, 0, 1, 0, 1, FILL, 0, 0, 0)
		label1.set_usize(-1, -1)
		label1.set_text("Summary")
		label1.set_justify(JUSTIFY_CENTER)
		label1.set_alignment(0, 0.5)
		label1.set_padding(0, 0)
		label1.show()
		self.label1=label1
		label2=GtkLabel()
		table1.attach(label2, 0, 1, 1, 2, FILL, 0, 0, 0)
		label2.set_usize(-1, -1)
		label2.set_text("Start")
		label2.set_justify(JUSTIFY_CENTER)
		label2.set_alignment(0, 0.5)
		label2.set_padding(0, 0)
		label2.show()
		self.label2=label2
		label3=GtkLabel()
		table1.attach(label3, 0, 1, 2, 3, FILL, 0, 0, 0)
		label3.set_usize(-1, -1)
		label3.set_text("End")
		label3.set_justify(JUSTIFY_CENTER)
		label3.set_alignment(0, 0.5)
		label3.set_padding(0, 0)
		label3.show()
		self.label3=label3
		label4=GtkLabel()
		table1.attach(label4, 0, 1, 3, 4, FILL, 0, 0, 0)
		label4.set_usize(-1, -1)
		label4.set_text("Alarm")
		label4.set_justify(JUSTIFY_CENTER)
		label4.set_alignment(0, 0.5)
		label4.set_padding(0, 0)
		label4.show()
		self.label4=label4
		label5=GtkLabel()
		table1.attach(label5, 0, 1, 4, 5, FILL, 0, 0, 0)
		label5.set_usize(-1, -1)
		label5.set_text("Description")
		label5.set_justify(JUSTIFY_CENTER)
		label5.set_alignment(0, 0.5)
		label5.set_padding(0, 0)
		label5.show()
		self.label5=label5

		summary_entry=GtkEntry()
		table1.attach(summary_entry, 1, 2, 0, 1, EXPAND, 0, 0, 0)
		summary_entry.set_flags(CAN_FOCUS)
		summary_entry.set_usize(160, -1)
		summary_entry.set_editable(TRUE)
		summary_entry.set_visibility(TRUE)
		summary_entry.set_max_length(0)
		if type == "edit":
			summary_entry.set_text(self.appt[self.selected_appointment][c_Summary])
		summary_entry.show()
		self.summary_entry=summary_entry

		start_time_hbox = GtkHBox()
		table1.attach(start_time_hbox, 1, 2, 1, 2, EXPAND|FILL, 0, 0, 0)
		start_time_hbox.show()

		start_time_combo_hour = GtkCombo()
		start_time_combo_hour.set_usize(45, -1)
		start_time_hbox.pack_start(start_time_combo_hour, expand=FALSE)
		t = 0
		start_time_combo_hour.set_popdown_strings(time_hours)
		if type == "edit":
			start_time_combo_hour.entry.set_text(self.appt[self.selected_appointment][c_StartTime][c_StartTimeHour])
		else:
			start_time_combo_hour.entry.set_text(self.cal_time[0])
		start_time_combo_hour.show()
		self.start_time_combo_hour = start_time_combo_hour

		start_time_combo_minute = GtkCombo()
		start_time_combo_minute.set_usize(45, -1)
		start_time_hbox.pack_start(start_time_combo_minute, expand=FALSE)
		start_time_combo_minute.set_popdown_strings(time_minutes)
		if type == "edit":
			start_time_combo_minute.entry.set_text(self.appt[self.selected_appointment][c_StartTime][c_StartTimeMin])
		else:
			start_time_combo_minute.entry.set_text(self.cal_time[1])
		start_time_combo_minute.show()
		self.start_time_combo_minute = start_time_combo_minute

		end_time_hbox = GtkHBox()
		table1.attach(end_time_hbox, 1, 2, 2, 3, EXPAND|FILL, 0, 0, 0)
		end_time_hbox.show()

		end_time_combo_hour = GtkCombo()
		end_time_combo_hour.set_usize(45, -1)
		end_time_hbox.pack_start(end_time_combo_hour, expand=FALSE)
		t = 0
		end_time_combo_hour.set_popdown_strings(time_hours)
		if type == "edit":
			end_time_combo_hour.entry.set_text(self.appt[self.selected_appointment][c_EndTime][c_EndTimeHour])
		else:
			end_time_combo_hour.entry.set_text(self.cal_time[0])
		end_time_combo_hour.show()
		self.end_time_combo_hour = end_time_combo_hour

		end_time_combo_minute = GtkCombo()
		end_time_combo_minute.set_usize(45, -1)
		end_time_hbox.pack_start(end_time_combo_minute, expand=FALSE)
		end_time_combo_minute.set_popdown_strings(time_minutes)
		if type == "edit":
			end_time_combo_minute.entry.set_text(self.appt[self.selected_appointment][c_EndTime][c_EndTimeMin])
		else:
			end_time_combo_minute.entry.set_text(self.cal_time[1])
		end_time_combo_minute.show()
		self.end_time_combo_minute = end_time_combo_minute

		alarm_check = GtkCheckButton()
		table1.attach(alarm_check, 1, 2, 3, 4, EXPAND|FILL, 0, 0, 0)
		if type == "edit":
			if self.appt[self.selected_appointment][c_Alarm][c_AlarmStatus] == 'yes':
				alarm_check.set_active(TRUE)
		if os.path.exists("/usr/bin/uschedule") == -1:
				alarm_check.set_sensitive(FALSE)
		alarm_check.show()
		self.alarm_check = alarm_check

		scrolledwindow1=GtkScrolledWindow()
		table1.attach(scrolledwindow1, 1, 2, 4, 5, EXPAND, 0, 0, 0)
		scrolledwindow1.set_usize(160, 70)
		scrolledwindow1.set_policy(POLICY_NEVER, POLICY_AUTOMATIC)
		description_text=GtkText()
		scrolledwindow1.add(description_text)
		description_text.set_flags(CAN_FOCUS)
		description_text.set_usize(160, 70)
		description_text.set_editable(TRUE)
		if type == "edit":
			description_text.insert(None,None,None,self.appt[self.selected_appointment][c_Description])
		description_text.show()
		self.description_text=description_text
		scrolledwindow1.show()
		self.scrolledwindow1=scrolledwindow1

	def select_date_from_calendar(self, *args):
		'''Select date from graphical calendar.'''

		# our main gtk window
		window = GtkWindow(WINDOW_TOPLEVEL)
		self.calendar_select_window = window
		window.connect("delete_event", window.hide)
		window.set_title("Storm - Calendar")
		window.set_usize(200, 140)
		window.set_name("window")
		rc_parse_string('widget "*window*" style "storm_normal"') 
		window.show()

		calendar = GtkCalendar()
		calendar.select_month(int(self.cal_date[c_DateMonth]) - 1, int(self.cal_date[c_DateYear]))
		calendar.select_day(int(self.cal_date[c_DateDay]))
		calendar.connect("day-selected-double-click", self.change_date)
		window.add(calendar)
		calendar.show()
		self.select_calendar = calendar

	def change_date(self, *args):
		'''Change date entries to the selected date.'''

		# Get the date from the graphical calendar
		year, month, day = self.select_calendar.get_date()
		# It gives us the months starting at 0, but we want them to start at 1
		month = month + 1

		# Make sure the numbers have a 0 on the front if needed
		self.cal_date[c_DateDay]   = string.rjust(str(day), 2).replace(' ', '0')
		self.cal_date[c_DateMonth] = string.rjust(str(month), 2).replace(' ', '0')
		self.cal_date[c_DateYear]  = str(year)

		# Set the text of our entry box
		self.date_label.set_text(time.strftime("%a, %b %d %Y", time.strptime(time.strftime("%b %d %Y", (int(year), int(month), int(day), 0, 0, 0, 0, 0, 0)), "%b %d %Y")))
		self.update_list()
		self.calendar_select_window.hide()

	def back_one_day(self, *args):
		'''Sets the selected date back by one day.'''

		secs = time.mktime((int(self.cal_date[c_DateYear]), int(self.cal_date[c_DateMonth]), int(self.cal_date[c_DateDay]), 0, 0, 0, 0, 0, 0))
		secs = secs - 86400
		date = time.strftime("%d %m %Y", time.localtime(secs))

		self.cal_date = string.split(date)
		self.date_label.set_text(time.strftime("%a, %b %d %Y", time.strptime(time.strftime("%b %d %Y", (int(self.cal_date[c_DateYear]), int(self.cal_date[c_DateMonth]), int(self.cal_date[c_DateDay]), 0, 0, 0, 0, 0, 0)), "%b %d %Y")))
		self.update_list()

	def forward_one_day(self, *args):
		'''Sets the selected date forward by one day.'''

		secs = time.mktime((int(self.cal_date[c_DateYear]), int(self.cal_date[c_DateMonth]), int(self.cal_date[c_DateDay]), 0, 0, 0, 0, 0, 0))
		secs = secs + 86400
		date = time.strftime("%d %m %Y", time.localtime(secs))

		self.cal_date = string.split(date)
		self.date_label.set_text(time.strftime("%a, %b %d %Y", time.strptime(time.strftime("%b %d %Y", (int(self.cal_date[c_DateYear]), int(self.cal_date[c_DateMonth]), int(self.cal_date[c_DateDay]), 0, 0, 0, 0, 0, 0)), "%b %d %Y")))
		self.update_list()
		
	def today_date(self, *args):
		'''Sets the selected date to today.'''

		date = time.strftime("%d %m %Y", time.localtime(time.time()))
		self.cal_date = string.split(date)

		self.date_label.set_text(time.strftime("%a, %b %d %Y", time.strptime(time.strftime("%b %d %Y", (int(self.cal_date[c_DateYear]), int(self.cal_date[c_DateMonth]), int(self.cal_date[c_DateDay]), 0, 0, 0, 0, 0, 0)), "%b %d %Y")))
		self.update_list()

		date_time = time.strftime("%H %M", time.localtime(time.time()))
		self.cal_time = string.split(date_time)

		time_row = int(self.cal_time[0]) * 2
		if int(self.cal_time[1]) >= 30:
			time_row = time_row + 1

		self.clist.select_row(time_row,0)

		vadjust = self.clist.get_vadjustment()
		vadjust.set_value((time_row * 16) - (16 * 3))

	def update_xml(self):
		'''Update the calendar xml storage file.'''

		file = open(self.parent.storm_userdatadir + 'appointments.xml', 'w')

		ApptIdx = 0
		while ApptIdx < len(self.appt):
			file.write('<appointment>\n<summary>' + self.appt[ApptIdx][c_Summary] + '</summary>\n<start-time-hour>' + self.appt[ApptIdx][c_StartTime][c_StartTimeHour] + '</start-time-hour>\n<start-time-minute>' + self.appt[ApptIdx][c_StartTime][c_StartTimeMin] + '</start-time-minute>\n<end-time-hour>' + self.appt[ApptIdx][c_EndTime][c_EndTimeHour] + '</end-time-hour>\n<end-time-minute>' + self.appt[ApptIdx][c_EndTime][c_StartTimeMin] + '</end-time-minute>\n<date-day>' + self.appt[ApptIdx][c_Date][c_DateDay] + '</date-day>\n<date-month>' + self.appt[ApptIdx][c_Date][c_DateMonth] + '</date-month>\n<date-year>' + self.appt[ApptIdx][c_Date][c_DateYear] + '</date-year>\n<alarm-status>' + self.appt[ApptIdx][c_Alarm][c_AlarmStatus] + '</alarm-status>\n<alarm-id>' + str(self.appt[ApptIdx][c_Alarm][c_AlarmId]) + '</alarm-id>\n<description>' + self.appt[ApptIdx][c_Description] + '</description>\n</appointment>\n\n')
			ApptIdx = ApptIdx + 1

		file.close
