#!/usr/bin/env python
# Storm mini Evolution clone by Damien Tanner (dctanner@xoasis.com)
#
# This file is part of Storm.
#
#    Storm is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Storm is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Storm; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from gtk import *
from modules.xmlparse import *
import string

class Notes:
	def __init__(self, parent):
		self.parent = parent

		frame=GtkFrame()
		self.frame = frame
		frame.set_usize(-1, -1)
		frame.set_shadow_type("GTK_SHADOW_ETCHED_IN")
		frame.set_label("Notes")
		frame.set_label_align(0, -1)
		frame.set_name("frame")
		rc_parse_string('widget "*frame*" style "storm_normal"') 
		frame.show()

		vpaned = GtkVPaned()
		frame.add(vpaned)
		vpaned.show()

		vbox = GtkVBox()
		vpaned.pack1(vbox)
		vbox.show()

		# make the toobar
		toolbar = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		toolbar.set_button_relief(RELIEF_NONE)
		toolbar.set_space_size(10)
		vbox.pack_start(toolbar, expand=FALSE)
		toolbar.show()
		# add the evolution pixmaps to the toolbar
		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/new.xpm")
		toolbar.append_item(None, 'Make a new note', None, GtkPixmap(pix, mask), self.new_note)

		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/save.xpm")
		toolbar.append_item(None, 'Save notes', None, GtkPixmap(pix, mask), self.save_notes)

		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/draw.xpm")
		toolbar.append_item(None, 'Add a drawing for the note', None, GtkPixmap(pix, mask), self.parent.scribble.new_scribble)

		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/trash.xpm")
		toolbar.append_item(None, 'Delete the selected note', None, GtkPixmap(pix, mask), self.delete_note)

		cscroll = GtkScrolledWindow()
		cscroll.set_policy(POLICY_AUTOMATIC, POLICY_AUTOMATIC)
		vbox.pack_start(cscroll)
		cscroll.show()

		clist=GtkCList(2)
		cscroll.add(clist)
		clist.connect("select_row" , self.select_note)
		clist.set_flags(CAN_FOCUS)
		clist.set_usize(-1, 87)
		clist.set_column_title(0, "Saved")
		clist.set_column_width(0, 16)
		clist.set_column_title(1, "Title")
		clist.set_column_width(1, 180)
		clist.set_selection_mode(SELECTION_SINGLE)
		clist.column_titles_hide()
		clist.show()
		self.clist=clist

		scroll = GtkScrolledWindow()
		scroll.set_policy(POLICY_NEVER, POLICY_ALWAYS)
		self.scroll = scroll
		vpaned.pack2(scroll)
		scroll.show()

		text = GtkText()
		text.connect("changed", self.text_changed)
		self.text = text
		text.set_editable(TRUE)
		scroll.add(text)
		text.show()

		self.update_list()

	def update_list(self):
		'''Updates th elsit of notes.'''

		# - Parse the xml file
		xml = XmlParse()
		notes_text = xml.parse_xml_file(self.parent.storm_userdatadir + 'notes.xml', 'note')

		self.note = []

		self.clist.freeze()

		for note_text in notes_text:

			# - Extract the first line as the title
			tmp = string.split(note_text, '\n', 1)

			if tmp[0] == '':
				note_title = '<no title>'
			else:
				note_title = tmp[0]

			# - Append the note to the list
			self.clist.append(['', note_title])
			self.note.append([0, note_text])

		self.clist.thaw()

	def select_note(self, *args):
		'''Called when a note is selected from the list.'''

		self.selected_note = args[1]

		# - Insert the notes text into the text box
		self.freeze_text = 1
		self.text.freeze()
		self.text.delete_text(0, -1)
		self.text.insert(None,None,None,self.note[self.selected_note][1])
		self.text.thaw()
		self.freeze_text = 0

		self.text.set_position(len(self.note[self.selected_note][1]))
		self.text.grab_focus()

	def new_note(self, *args):
		'''Makes a new note.'''

		# - Append the blank note title to the list
		self.clist.append(['', '<no title>'])

		# - Add the real new blank note data
		self.note.append([0, ''])
		self.selected_note = len(self.note) - 1
		
		# - Add a not saved icon since the note is new
		pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/save-tiny-red.xpm")
		pixmap = GtkPixmap(pix, mask)
		self.clist.set_pixmap(self.selected_note, 0, pixmap)

		# - Selected the note
		self.clist.select_row(self.selected_note, 1)

		# - Grab the text boxes focus so we can start entrting data
		self.text.grab_focus()

	def delete_note(self, *args):
		'''Delete the currently selected note.'''

		# - Delete the note data
		del self.note[self.selected_note]

		# - Remove it from the list
		self.clist.remove(self.selected_note)

		# - Delete any existing text box text
		self.freeze_text = 1
		self.text.delete_text(0, -1)
		self.freeze_text = 0

		# - Save the rest of the notes
		self.save_notes()

	def text_changed(self, *args):
		'''Updates the notes title when the text is edited.'''

		# - If we're not just inserting text from a note being selected
		if self.freeze_text == 0:

			# - Get the notes text
			txt = self.text.get_chars(0, -1)

			# - Extract the first line as the title
			tmp = string.split(txt, '\n', 1)

			# - Refresh the next for the notes list
			self.clist.set_text(self.selected_note, 1, tmp[0])

			# - Update the note with the new data
			self.note[self.selected_note][1] = txt

			# - See if the note has been saved or not
			if self.note[self.selected_note][0] == 0:
				self.note[self.selected_note][0] = 1

				# - Change the pixmap
				pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/save-tiny-red.xpm")
				pixmap = GtkPixmap(pix, mask)
				self.clist.set_pixmap(self.selected_note, 0, pixmap)
				
	def save_notes(self, *args):
		'''Saves edited notes.'''

		# - Open up the xml file
		file = open(self.parent.storm_userdatadir + 'notes.xml', 'w')

		# - Write each note to it
		x = 0
		while x < len(self.note):
			file.write('<note>' + self.note[x][1] + '</note>\n\n')
			
			if self.note[x][0] == 1:
				self.note[x][0] = 0

				# - Change the pixmap
				pix, mask = create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/blank-16.xpm")
				pixmap = GtkPixmap(pix, mask)
				self.clist.set_pixmap(x, 0, pixmap)
			x = x + 1
		file.close()
