#!/usr/bin/env python
# Storm mini Evolution clone by Damien Tanner (dctanner@xoasis.com)
#
# This file is part of Storm.
#
#    Storm is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Storm is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Storm; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import gtk, GDK

class Scribble:
	def __init__(self, parent):
		self.parent = parent
		
	def new_scribble(self, *args):
		'''Open a new window for drawing.'''

		window = gtk.GtkWindow(gtk.WINDOW_TOPLEVEL)
		self.scribble_window = window
		window.connect("delete_event", window.hide)
		window.set_title("Storm - Scribble")
		window.set_usize(240, 240)
		window.set_name("window")
		gtk.rc_parse_string('widget "*window*" style "storm_normal"') 
		window.show()

		vbox = gtk.GtkVBox()
		window.add(vbox)
		vbox.show()

		hbox = gtk.GtkHBox()
		vbox.pack_start(hbox, expand=gtk.FALSE)
		hbox.show()

		toolbar = gtk.GtkToolbar(gtk.ORIENTATION_HORIZONTAL, gtk.TOOLBAR_ICONS)
		toolbar.set_button_relief(gtk.RELIEF_NONE)
		toolbar.set_space_size(10)
		hbox.pack_start(toolbar, expand=gtk.FALSE)
		toolbar.show()

		pix, mask = gtk.create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/save.xpm")
		toolbar.append_item(None, 'Save Drawing', None, gtk.GtkPixmap(pix, mask), self.save_drawing)

		pix, mask = gtk.create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/cancel.xpm")
		toolbar.append_item(None, 'Close this window', None, gtk.GtkPixmap(pix, mask), window.hide)

		toolbar2 = gtk.GtkToolbar(gtk.ORIENTATION_HORIZONTAL, gtk.TOOLBAR_ICONS)
		toolbar2.set_button_relief(gtk.RELIEF_NONE)
		toolbar2.set_space_size(10)
		hbox.pack_end(toolbar2, expand=gtk.FALSE)
		toolbar2.show()

		pix, mask = gtk.create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/pencil.xpm")
		toolbar2.append_item(None, 'Pencil', None, gtk.GtkPixmap(pix, mask), self.use_pencil)

		pix, mask = gtk.create_pixmap_from_xpm(self.parent.window, None, self.parent.config[2] + "/pixmaps/eraser.xpm")
		toolbar2.append_item(None, 'Eraser', None, gtk.GtkPixmap(pix, mask), self.use_eraser)

		drawing_area = gtk.GtkDrawingArea()

		# - Signals used to handle backing pixmap
		drawing_area.connect("expose_event", self.expose_event)
		drawing_area.connect("configure_event", self.configure_event)

		# - Event signals
		drawing_area.connect("motion_notify_event", self.motion_notify_event)
		drawing_area.connect("button_press_event", self.button_press_event)

		drawing_area.set_events(GDK.EXPOSURE_MASK | GDK.LEAVE_NOTIFY_MASK | GDK.BUTTON_PRESS_MASK | GDK.POINTER_MOTION_MASK | GDK.POINTER_MOTION_HINT_MASK)

		drawing_area.size(200, 200)
		vbox.pack_start(drawing_area)
		drawing_area.show()

		# - Backing pixmap for drawing area
		self.pixmap = None
		self.color = "black"

	def use_pencil(self, *args):
		self.color = "black"

	def use_eraser(self, *args):
		self.color = "white"

	def configure_event(self, widget, event):
		'''Create a new backing pixmap of the appropriate size.'''

		x, y, width, height = widget.get_allocation()
		self.pixmap = gtk.create_pixmap(widget.get_window(), width, height, -1)
		gtk.draw_rectangle(self.pixmap, widget.get_style().white_gc, gtk.TRUE, 0, 0, width, height)

		return gtk.TRUE

	def expose_event(self, widget, event):
		'''Redraw the screen from the backing pixmap.'''

		x , y, width, height = event.area
		widget.draw_pixmap(widget.get_style().fg_gc[gtk.STATE_NORMAL], self.pixmap, x, y, x, y, width, height)

		return gtk.FALSE

	def draw_brush(self, widget, x, y):
		'''Draw a rectangle on the screen.'''

		if self.color == "black":
			rect = (x - 1, y - 1, 2, 2)
			gtk.draw_rectangle(self.pixmap, widget.get_style().black_gc, gtk.TRUE, rect[0], rect[1], rect[2], rect[3])
		elif self.color == "white":
			rect = (x - 5, y - 5, 10, 10)
			gtk.draw_rectangle(self.pixmap, widget.get_style().white_gc, gtk.TRUE, rect[0], rect[1], rect[2], rect[3])
		widget.draw(rect)

	def button_press_event(self, widget, event):
		if event.button == 1 and self.pixmap != None:
			self.draw_brush(widget, event.x, event.y)

		return gtk.TRUE

	def motion_notify_event(self, widget, event):
		if event.is_hint:
			x, y, = event.window.pointer
			state = event.window.pointer_state
		else:
			x = event.x
			y = event.y
			state = event.state

		if state & GDK.BUTTON1_MASK and self.pixmap != None:
			self.draw_brush(widget, x, y)

		return gtk.TRUE

	def save_drawing(self, *args):
		pass
