#!/usr/bin/env python
# Storm mini Evolution clone by Damien Tanner (dctanner@xoasis.com)      
#
# This file is part of Storm.
#
#    Storm is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Storm is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.

#    You should have received a copy of the GNU General Public License
#    along with Storm; if not, write to the Free Software
#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from gtk import *
import string, os, sys
from modules.stormload import *
from modules.mail import *
from modules.today import *
from modules.notes import *
from modules.contacts import *
from modules.calendar import *
from modules.help import *
from modules.sync import *
from modules.prefs import *
from modules.tasks import *
from modules.scribble import *
from modules.storm_constants import *

# main class for the gui, other classes handle diffrent sub functions such as Mail and Contacts
class Storm:
	def __init__(self):
		global first_time

		self.VERSION = "0.5.2"

		# directories and files for config and data:
		self.storm_userdatafile    = os.environ['HOME'] + '/.stormrc'
		self.storm_usercfgdir      = os.environ['HOME'] + '/.storm/config/'
		self.storm_userdatadir     = os.environ['HOME'] + '/.storm/data/'
		self.storm_usermaildirdir  = os.environ['HOME'] + '/Maildir/'

		# read storm config file
		self.get_config()

		# some locations where gtkrc's could be...
                rc_parse("/etc/gt2k/gtkrc")
                rc_parse("/usr/share/gtkrc")
                rc_parse(os.environ['HOME'] + "/.gtkrc")
                rc_parse(self.config[2] + "/stormgtkrc")
                rc_parse(os.environ['HOME'] + "/.stormgtkrc")
		rc_parse_string('widget "*" style "storm_normal"')

		# our main gtk window
		self.window = GtkWindow(WINDOW_TOPLEVEL)
		self.window.connect("delete_event", self.quit)
		self.window.set_title("Storm v" + self.VERSION)
		if screen_width() > 300:
			self.window.set_usize(280, 240)
		else:
			self.window.set_usize(240, 280)

		# show a nice loading popup
		self.loading = StormLoad(self)
		while events_pending(): mainiteration()
		# start by importing the other classes
		self.scribble = Scribble(self)
		self.contacts = Contacts(self)
		while events_pending(): mainiteration()
		self.loading.today_pixmap.show()
		self.mail = Mail(self)
		while events_pending(): mainiteration()
		self.loading.inbox_pixmap.show()
		self.calendar = Calendar(self)
		while events_pending(): mainiteration()
		self.loading.calendar_pixmap.show()
		self.tasks = Tasks(self)
		self.notes = Notes(self)
		while events_pending(): mainiteration()
		self.loading.contacts_pixmap.show()
		self.sync = Sync(self)
		self.prefs = Prefs(self)
		while events_pending(): mainiteration()
		self.loading.notes_pixmap.show()
		self.today = Today(self)
		self.help = Help(self)
		while events_pending(): mainiteration()
		self.loading.tasks_pixmap.show()
		
		# close the loading window
		self.loading.window.hide()
		
		# Main vBox
		if screen_width() > 300:
			self.vbox = GtkHBox()
			self.window.add(self.vbox)
			self.vbox.show()

			hbox = GtkVBox()
			self.vbox.pack_start(hbox, expand=FALSE)
			hbox.show()
		else:
			self.vbox = GtkVBox()
			self.window.add(self.vbox)
			self.vbox.show()

			hbox = GtkHBox()
			self.vbox.pack_start(hbox, expand=FALSE)
			hbox.show()

		# make the toobar
		if screen_width() > 300:
			self.toolbar = GtkToolbar(ORIENTATION_VERTICAL, TOOLBAR_ICONS)
		else:
			self.toolbar = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		self.toolbar.set_button_relief(RELIEF_NONE)
		self.toolbar.set_space_size(10)
		hbox.pack_start(self.toolbar, expand=TRUE)
		self.toolbar.show()

		# make our notbook
		self.notebook = GtkNotebook()
		self.notebook.show()
		self.notebook.set_show_tabs(FALSE)
		self.notebook.set_show_border(FALSE)
		# add it to the vbox
		self.vbox.pack_start(self.notebook)
		# add a load of pages to it
		self.notebook.add(self.today.frame)
		self.notebook.add(self.mail.frame)
		self.notebook.add(self.calendar.frame)
		self.notebook.add(self.tasks.frame)
		self.notebook.add(self.contacts.frame)
		self.notebook.add(self.notes.frame)

		# add the evolution pixmaps to the toolbar
		pix, mask = create_pixmap_from_xpm(self.window, None, self.config[2] + "/pixmaps/storm-today-mini.xpm")
		self.toolbar.append_item(None, 'Today', None, GtkPixmap(pix, mask), self.show_today)

		pix, mask = create_pixmap_from_xpm(self.window, None, self.config[2] + "/pixmaps/storm-inbox-mini.xpm")
		self.toolbar.append_item(None, 'E-Mail', None, GtkPixmap(pix, mask), self.show_mail)

		pix, mask = create_pixmap_from_xpm(self.window, None, self.config[2] + "/pixmaps/storm-calendar-mini.xpm")
		self.toolbar.append_item(None, 'Calendar', None, GtkPixmap(pix, mask), self.show_calendar)

		pix, mask = create_pixmap_from_xpm(self.window, None, self.config[2] + "/pixmaps/storm-tasks-mini.xpm")
		self.toolbar.append_item(None, 'Tasks', None, GtkPixmap(pix, mask), self.show_tasks)

		pix, mask = create_pixmap_from_xpm(self.window, None, self.config[2] + "/pixmaps/storm-contacts-mini.xpm")
		self.toolbar.append_item(None, 'Contacts', None, GtkPixmap(pix, mask), self.show_contacts)

		pix, mask = create_pixmap_from_xpm(self.window, None, self.config[2] + "/pixmaps/storm-notes-mini.xpm")
		self.toolbar.append_item(None, 'Notes', None, GtkPixmap(pix, mask), self.show_notes)

		# make another toolbar for other items such as prefs, about and sync
		if screen_width() > 300:
			self.toolbar2 = GtkToolbar(ORIENTATION_VERTICAL, TOOLBAR_ICONS)
		else:
			self.toolbar2 = GtkToolbar(ORIENTATION_HORIZONTAL, TOOLBAR_ICONS)
		self.toolbar2.set_button_relief(RELIEF_NONE)
		self.toolbar2.set_space_size(10)
		hbox.pack_end(self.toolbar2, expand=FALSE)
		self.toolbar2.show()

		pix, mask = create_pixmap_from_xpm(self.window, None, self.config[2] + "/pixmaps/prefs.xpm")
		self.toolbar2.append_item(None, 'Preferences', None, GtkPixmap(pix, mask), self.prefs.show_config)

		self.sync_menubar = GtkMenuBar()
		self.sync_menubar.set_shadow_type(SHADOW_NONE)
		pix, mask = create_pixmap_from_xpm(self.window, None, self.config[2] + "/pixmaps/send-receive.xpm")

		sync_menuitem = GtkMenuItem()
		sync_pixmap = GtkPixmap(pix,mask)
		sync_menuitem.add(sync_pixmap)
		sync_pixmap.show()
		sync_menuitem.show()
		self.sync_menuitem = sync_menuitem
		self.sync_menubar.append(sync_menuitem)
		self.toolbar2.append_widget(self.sync_menubar, 'Sync', '')
		self.sync_menubar.show()

		sync_menu = GtkMenu()

		sync_get_mail_menuitem = GtkMenuItem("Fetch Messages")
		sync_get_mail_menuitem.connect("activate", self.sync.get_mail)
		sync_menu.append(sync_get_mail_menuitem)
		sync_get_mail_menuitem.show()

		sync_send_mail_menuitem = GtkMenuItem("Send Messages")
		sync_send_mail_menuitem.connect("activate", self.sync.send_mail)
		sync_menu.append(sync_send_mail_menuitem)
		sync_send_mail_menuitem.show()

		sync_menu.show()
		sync_menuitem.set_submenu(sync_menu)

		pix, mask = create_pixmap_from_xpm(self.window, None, self.config[2] + "/pixmaps/exit.xpm")
		self.toolbar2.append_item(None, 'Exit', None, GtkPixmap(pix, mask), self.quit)

		# check every c_TimeoutCheckTime ms:
		self.timer = timeout_add(c_TimeoutCheckTime, self.today_timeout)

	def show_today(self, *args):
		self.today.update_today()
		self.notebook.set_page(0)

	def show_mail(self, *args):
		self.notebook.set_page(1)

	def show_calendar(self, *args):
		self.notebook.set_page(2)
		self.calendar.today_date()

	def show_tasks(self, *args):
		self.notebook.set_page(3)

	def show_contacts(self, *args):
		self.notebook.set_page(4)

	def show_notes(self, *args):
		self.notebook.set_page(5)

	def get_config(self):
		'''Get the ~/.stormrc global config.'''

		# If there's no ~/.stormrc we must be running it for
		# the first time, so make one and also launch config
		self.first_time = 0

		if os.path.isfile(self.storm_userdatafile) <= 0:
			# Make a new ~/.stormrc with some defaults
			print "DEBUG: ~/.stormrc not existing, generating it..."
			file = open(self.storm_userdatafile, 'w')
			# lines meaning: 1) the user name
			#                2) the user e-mail
			#                3) the directory where stuff like pixmaps are pulled from
			#                4) (not used?)
			file.write('Your Name\nyour.email@mydomain.com\n/usr/share/storm/\nno\n')
			file.close()
			print "DEBUG: ~/.stormrc not existing, generating it...done"

			self.first_time == 1

		if os.path.isfile(self.storm_usercfgdir + 'lastsync') <= 0:
			file = open(self.storm_usercfgdir + 'lastsync', 'w')
			file.write('<never>')
			file.close()

		# Open up the storm config file
		file = open(self.storm_userdatafile, 'r')
		# read it
		lines = file.readlines()

		if lines < 4:
			print "Your file '%s' is corrupted. Exiting." % self.storm_userdatafile
			sys.exit(1)
		
		file.close()
		# assign variables to the different lines
		self.config = []
		for line in lines:
			line = string.strip(line)
			self.config.append(line)

		# check if the template dir exists:
		templatedir = self.config[2] + '/data/templates/'
		print "DEBUG: templatedir is '%s'" % templatedir
		if not os.path.exists(templatedir):
			print "DEBUG: No template directory '%s' found! Exiting." % templatedir
			sys.exit(1)
		# copy the data template files:
		if not os.path.exists(self.storm_userdatadir):
			for template in os.listdir(templatedir):
				print "DEBUG: Copying data template files..."
				if os.path.isfile(self.storm_userdatadir + template) <= 0:
					open(self.storm_userdatadir + template, 'w').write(open(self.config[2] + '/data/templates/' + template).read())
				print "DEBUG: Copying data template files...done"
					
		# copy the config template files:
		if not os.path.exists(self.storm_usercfgdir):
			for template in os.listdir(self.config[2] + '/config/templates/'):
				print "DEBUG: Copying config template files..."
				if os.path.isfile(self.storm_usercfgdir + template) <= 0:
					open(self.storm_usercfgdir + template, 'w').write(open(self.config[2] + '/config/templates/' + template).read())
				print "DEBUG: Copying config template files...done"

		# copy the mail welcome message:
		if self.first_time == 1:
			print "DEBUG: Copying mail welcome message..."
			if os.path.exists(self.storm_usermaildirdir + "Inbox/new/testmessage") <= 0:
				open(self.storm_usermaildirdir + "Inbox/new/testmessage", 'w').write(open(self.config[2] + '/maildir/templates/testmessage').read())
			print "DEBUG: Copying mail welcome message...done"
		
	def new_note(self, *args):
		self.show_notes()
		self.notes.new_note()

	def mainloop(self):
		self.window.show()
		if self.first_time == 1:
			self.prefs.show_config()
		mainloop()

	def quit(self, *args):
		# quit storm
		timeout_remove(self.timer)
                mainquit()

	def today_timeout(self):
		cursecs = time.strftime("%S", time.localtime(time.time()))
		# c_TimeoutCheckTime is in millisecs:
		if cursecs <= c_TimeoutCheckTime/1000:
			self.today.update_today_datetime()
		# always return TRUE if the timeout should stay there:
		return TRUE

storm = Storm()
storm.mainloop()

# new idea for lock file handling: catch all signals and remove the lock before dying
#
# def handler(signum, frame):
# 	# remove lock file (which should be in the user's $HOME and contain the PID) here
# 	signal.signal(signum, signal.SIG_DFL)
# 
# siglist = []
# for s,v in signal.__dict__.items():
# 	if s[0:3] == "SIG" and not s[3] == "_":
# 	#if s[0:3] == "SIG" and not s[3] == "_" and not s == "SIGSTOP" and not s == "SIGTSTP":
# 		print "s = ", s
# 		print "v = ", v
# 		siglist.append(v)
# 
# for i in siglist:
# 	signal.signal(i, handler)

# old lock file handling:
#
# if os.path.exists("/tmp/storm.lock") <= 0:
# 	os.system("touch /tmp/storm.lock")
# 	storm = Storm()
# 	storm.mainloop()
# else:
# 	print "Storm already seems to be running."
# 	print "Please delete '/tmp/storm.lock' manually if this is not true."
